@@ -28,6 +28,8 @@
 #include "gtkhtml.h"
 #include "gtkhtml-embedded.h"
 #include "htmlembedded.h"
+#include "htmlframe.h"
+#include "htmliframe.h"
 #include "htmlpainter.h"
 #include "htmlengine.h"
 
@@ -68,7 +70,7 @@ draw (HTMLObject *o,
 	if (!element->widget)
 		return;
 
-	if (element->widget) {
+	if (element->parent) {
 		new_x = GTK_LAYOUT (element->parent)->hadjustment->value + o->x + tx;
 		new_y = GTK_LAYOUT (element->parent)->vadjustment->value + o->y + ty - o->ascent;
 		
@@ -78,8 +80,10 @@ draw (HTMLObject *o,
 			else
 				gtk_widget_queue_draw (element->widget);
 		}
+	
 		element->abs_x = new_x;
 		element->abs_y = new_y;
+		
 		if (!element->widget->parent)
 			gtk_layout_put (GTK_LAYOUT(element->parent), element->widget, new_x, new_y);
 	}
@@ -100,9 +104,15 @@ destroy (HTMLObject *o)
 		g_free(element->value);
 	if(element->widget) {
 		gtk_widget_hide (element->widget);
+		gtk_signal_disconnect_by_data (GTK_OBJECT (element->widget), element);
+		if (element->changed_id > 0)
+			gtk_signal_disconnect (GTK_OBJECT (element->widget), element->changed_id);
+		gtk_object_set_data (GTK_OBJECT (element->widget), "embeddedelement", NULL);
 		if (element->widget->parent && element->parent) {
 			g_assert (element->widget->parent == element->parent);
 			gtk_container_remove (GTK_CONTAINER (element->parent), element->widget);
+		} else {
+		       	gtk_object_sink (GTK_OBJECT (element->widget));
 		}
 	}
 
@@ -138,8 +148,7 @@ calc_min_width (HTMLObject *self,
 }
 
 static gboolean
-calc_size (HTMLObject *self,
-	   HTMLPainter *painter)
+calc_size (HTMLObject *self, HTMLPainter *painter, GList **changed_objs)
 {
 	GtkWidget *widget;
 	HTMLEmbedded *emb = HTML_EMBEDDED (self);
@@ -173,11 +182,14 @@ calc_size (HTMLObject *self,
 	return FALSE;
 }
 
-
-void
-html_embedded_reset (HTMLEmbedded *e)
+static void
+reparent (HTMLEmbedded *e, GtkWidget *new_parent)
 {
-	HTML_EMBEDDED_CLASS (HTML_OBJECT (e)->klass)->reset (e);
+	e->parent = new_parent;
+	gtk_widget_ref (e->widget);
+	gtk_widget_unparent (e->widget);
+	gtk_layout_put (GTK_LAYOUT(e->parent), e->widget, 0, 0);
+	gtk_widget_unref (e->widget);
 }
 
 static gchar *
@@ -185,6 +197,12 @@ encode (HTMLEmbedded *e)
 {
 	return g_strdup ("");
 }
+
+void
+html_embedded_reset (HTMLEmbedded *e)
+{
+	HTML_EMBEDDED_CLASS (HTML_OBJECT (e)->klass)->reset (e);
+}
 
 gchar *
 html_embedded_encode (HTMLEmbedded *e)
@@ -192,6 +210,12 @@ html_embedded_encode (HTMLEmbedded *e)
 	return HTML_EMBEDDED_CLASS (HTML_OBJECT (e)->klass)->encode (e);
 }
 
+void
+html_embedded_reparent (HTMLEmbedded *e, GtkWidget *new_parent)
+{
+	HTML_EMBEDDED_CLASS (HTML_OBJECT (e)->klass)->reparent (e, new_parent);
+}
+
 void
 html_embedded_set_form (HTMLEmbedded *e, HTMLForm *form)
 {
@@ -263,6 +287,7 @@ html_embedded_class_init (HTMLEmbeddedClass *klass,
 	/* HTMLEmbedded methods.   */
 	klass->reset = reset;
 	klass->encode = encode;
+	klass->reparent = reparent;
 
 	/* HTMLObject methods.   */
 	object_class->destroy = destroy;
@@ -300,8 +325,10 @@ html_embedded_init (HTMLEmbedded *element,
 	element->width  = 0;
 	element->height = 0;
 	element->abs_x  = element->abs_y = -1;
+	element->changed_id = 0;
 }
 
+
 static gboolean
 html_embedded_grab_cursor(GtkWidget *eb, GdkEvent *event)
 {
@@ -309,8 +336,21 @@ html_embedded_grab_cursor(GtkWidget *eb, GdkEvent *event)
 	return TRUE;
 }
 
+/* called when some state in an embedded html object has changed ... do a redraw */
+static void
+html_embedded_object_changed (GtkHTMLEmbedded *eb, HTMLEngine *e)
+{
+	HTMLObject *object;
+
+	object = gtk_object_get_data(GTK_OBJECT(eb), "embeddedelement");
+	if (object)
+		html_object_calc_size (object, e->painter, FALSE);
+	
+	html_engine_schedule_update(e);
+}
+
 HTMLEmbedded *
-html_embedded_new_widget (GtkWidget *parent, GtkHTMLEmbedded *eb)
+html_embedded_new_widget (GtkWidget *parent, GtkHTMLEmbedded *eb, HTMLEngine *engine)
 {
 	HTMLEmbedded *em;
 
@@ -318,18 +358,23 @@ html_embedded_new_widget (GtkWidget *parent, GtkHTMLEmbedded *eb)
 	html_embedded_init (em, HTML_EMBEDDED_CLASS (&html_embedded_class), parent, eb->name, "");
 	html_embedded_set_widget (em, GTK_WIDGET (eb));
 
+	/* pass em as the user_data so that the handler will disconnect 
+	 * when the object is destoyed
+	 */
 	gtk_signal_connect(GTK_OBJECT(eb), "button_press_event",
-			   GTK_SIGNAL_FUNC(html_embedded_grab_cursor), NULL);
+			   GTK_SIGNAL_FUNC(html_embedded_grab_cursor), em);
+	em->changed_id = gtk_signal_connect (GTK_OBJECT (eb), "changed", html_embedded_object_changed, engine);
+	/* printf ("id %u\n", em->changed_id); */
 
 	return em;
 }
 
 static void
-allocate (GtkWidget *w, GtkAllocation  *allocation, HTMLEmbedded *e)
+html_embedded_allocate (GtkWidget *w, GtkAllocation  *allocation, HTMLEmbedded *e)
 {
 	if (e->width != allocation->width || e->height != allocation->height) {
 		if (e->width != allocation->width) {
-			html_object_change_set (HTML_OBJECT (e), HTML_CHANGE_ALL);			
+			html_object_change_set (HTML_OBJECT (e), HTML_CHANGE_ALL_CALC);
 			e->width = allocation->width;
 		}
 		e->height = allocation->height;
@@ -345,10 +390,10 @@ html_embedded_set_widget (HTMLEmbedded *emb, GtkWidget *w)
 	emb->widget = w;
 	
 	gtk_widget_show (w);
-
+	
 	gtk_object_set_data (GTK_OBJECT (w), "embeddedelement", emb);
 	gtk_signal_connect (GTK_OBJECT (w), "size_allocate",
-			    GTK_SIGNAL_FUNC (allocate), emb);
+			    GTK_SIGNAL_FUNC (html_embedded_allocate), emb);
 }
 
 GtkWidget *
@@ -357,3 +402,33 @@ html_embedded_get_widget (HTMLEmbedded *e)
 	return e->widget;
 }
 
+gboolean
+html_object_is_embedded (HTMLObject *o)
+{
+	gboolean rv = FALSE;
+
+	switch (o->klass->type) {
+	case HTML_TYPE_EMBEDDED:
+	case HTML_TYPE_TEXTINPUT:
+	case HTML_TYPE_BUTTON:
+	case HTML_TYPE_IMAGEINPUT:
+	case HTML_TYPE_TEXTAREA:
+	case HTML_TYPE_HIDDEN:
+	case HTML_TYPE_RADIO:
+	case HTML_TYPE_CHECKBOX:
+	case HTML_TYPE_SELECT:
+	case HTML_TYPE_IFRAME:
+	case HTML_TYPE_FRAME:
+		rv = TRUE;
+	default:
+		;
+	}
+
+	return rv;
+}
+
+gboolean
+html_object_is_frame (HTMLObject *o)
+{
+	return HTML_IS_FRAME (o) || HTML_IS_IFRAME (o);
+}
