@@ -30,18 +30,22 @@
 #include "gtkhtml-properties.h"
 
 #include "htmlclue.h"
+#include "htmlclueflow.h"
 #include "htmlcursor.h"
 #include "htmlcolorset.h"
 #include "htmlengine.h"
 #include "htmlengine-edit.h"
+#include "htmlengine-edit-clueflowstyle.h"
 #include "htmlengine-edit-cursor.h"
 #include "htmlengine-edit-cut-and-paste.h"
 #include "htmlengine-edit-fontstyle.h"
 #include "htmlengine-edit-movement.h"
 #include "htmlengine-edit-selection-updater.h"
+#include "htmlimage.h"
 #include "htmlinterval.h"
 #include "htmllinktext.h"
 #include "htmlobject.h"
+#include "htmlplainpainter.h"
 #include "htmltable.h"
 #include "htmlselection.h"
 #include "htmlsettings.h"
@@ -55,6 +59,8 @@ static void        insert_object (HTMLEngine *e, HTMLObject *obj, guint len, gui
 				  HTMLUndoDirection dir, gboolean check);
 static void        append_object (HTMLEngine *e, HTMLObject *o, guint len, HTMLUndoDirection dir);
 static void        insert_empty_paragraph (HTMLEngine *e, HTMLUndoDirection dir, gboolean add_undo);
+static void        insert_setup_undo (HTMLEngine *e, guint len, guint position_before, HTMLUndoDirection dir,
+				      gboolean delete_paragraph_before, gboolean delete_paragraph_after);
 
 /* helper functions -- need refactor */
 
@@ -220,6 +226,7 @@ remove_empty_and_merge (HTMLEngine *e, gboolean merge, GList *left, GList *right
 	HTMLObject *lo, *ro, *prev;
 
 #ifdef OP_DEBUG
+	HTMLObject *left_orig = left->data;
 	printf ("before merge\n");
 	gtk_html_debug_dump_tree_simple (e->clue, 0);
 	if (left && left->data) {
@@ -284,6 +291,8 @@ remove_empty_and_merge (HTMLEngine *e, gboolean merge, GList *left, GList *right
 		e->cursor->offset = html_object_get_length (e->cursor->object);
 	}
 #ifdef OP_DEBUG
+	printf ("-- finished\n");
+	gtk_html_debug_dump_tree_simple (left_orig, 0);
 	printf ("-- after\n");
 	gtk_html_debug_dump_tree_simple (e->clue, 0);
 	printf ("-- END merge\n");
@@ -303,6 +312,15 @@ split_and_add_empty_texts (HTMLEngine *e, gint level, GList **left, GList **righ
 #ifdef OP_DEBUG
 	printf ("-- SPLIT middle\n");
 	gtk_html_debug_dump_tree_simple (e->clue, 0);
+	printf ("-- SPLIT finish\n");
+	if (*left && (*left)->data) {
+		printf ("left\n");
+		gtk_html_debug_dump_tree_simple (HTML_OBJECT ((*left)->data), 0);
+	}
+	if (*right && (*right)->data) {
+		printf ("right\n");
+		gtk_html_debug_dump_tree_simple (HTML_OBJECT ((*right)->data), 0);
+	}
 	printf ("-- SPLIT end\n");
 #endif
 }
@@ -314,15 +332,18 @@ html_engine_copy_object (HTMLEngine *e, HTMLObject **o, guint *len)
 {
 	GList *from, *to;
 
-	if (html_engine_is_selection_active (e)) {
+	if (e->clue && HTML_CLUE (e->clue)->head && html_engine_is_selection_active (e)) {
 		prepare_delete_bounds (e, &from, &to, NULL, NULL);
 		*len = 0;
-		*o    = html_object_op_copy (HTML_OBJECT (from->data), e,
+		*o    = html_object_op_copy (HTML_OBJECT (from->data), NULL, e,
 					     from->next, to->next, len);
 #ifdef OP_DEBUG
 		printf ("copy len: %d (parent %p)\n", *len, (*o)->parent);
 		gtk_html_debug_dump_tree_simple (*o, 0);
 #endif
+	} else {
+		*len = 0;
+		*o = NULL;
 	}
 }
 
@@ -358,7 +379,7 @@ delete_undo_action (HTMLEngine *e, HTMLUndoData *data, HTMLUndoDirection dir, gu
 	guint       len = 0;
 
 	undo         = (DeleteUndo *) data;
-	buffer       = html_object_op_copy (undo->buffer, e, NULL, NULL, &len);
+	buffer       = html_object_op_copy (undo->buffer, NULL, e, NULL, NULL, &len);
 	insert_object (e, buffer, undo->buffer_len, position_after, undo->level, html_undo_direction_reverse (dir), TRUE);
 }
 
@@ -378,7 +399,7 @@ delete_setup_undo (HTMLEngine *e, HTMLObject *buffer, guint len, guint position_
 	/* printf ("delete undo len %d\n", len); */
 
 	html_undo_add_action (e->undo,
-			      html_undo_action_new ("Delete", delete_undo_action,
+			      html_undo_action_new ("Delete object", delete_undo_action,
 						    HTML_UNDO_DATA (undo), html_cursor_get_position (e->cursor),
 						    position_after),
 			      dir);
@@ -413,8 +434,88 @@ place_cursor_before_mark (HTMLEngine *e)
 	}
 }
 
+static gboolean
+haligns_equal (HTMLHAlignType a1, HTMLHAlignType a2)
+{
+	return a1 == a2
+		|| (a1 == HTML_HALIGN_LEFT && a2 == HTML_HALIGN_NONE)
+		|| (a1 == HTML_HALIGN_NONE && a2 == HTML_HALIGN_LEFT);
+
+}
+
+static gboolean
+levels_equal (HTMLClueFlow *me, HTMLClueFlow *you)
+{
+	if (!you)
+		return FALSE;
+
+	if (me->levels->len != you->levels->len)
+		return FALSE;
+
+	if (me->levels->len == 0)
+		return TRUE;
+
+	return !memcmp (me->levels->data, you->levels->data, you->levels->len);
+}
+
+static void
+check_flows (HTMLEngine *e, HTMLUndoDirection dir)
+{
+	/* I assume here that cursor is before mark */
+	HTMLClueFlow *flow1, *flow2;
+	gint level1, level2;
+
+	g_return_if_fail (e->cursor);
+	g_return_if_fail (e->cursor->object);
+	g_return_if_fail (e->cursor->object->parent);
+	g_return_if_fail (e->mark);
+	g_return_if_fail (e->mark->object);
+	g_return_if_fail (e->mark->object->parent);
+	g_return_if_fail (e->cursor->position <= e->mark->position);
+
+	if (!HTML_IS_CLUEFLOW (e->cursor->object->parent) || !HTML_IS_CLUEFLOW (e->mark->object->parent))
+		return;
+
+	level1 = html_object_get_parent_level (e->cursor->object->parent);
+	level2 = html_object_get_parent_level (e->mark->object->parent);
+
+	flow1 = HTML_CLUEFLOW (e->cursor->object->parent);
+	flow2 = HTML_CLUEFLOW (e->mark->object->parent);
+
+	if (level1 == level2
+	    && (flow1->style != flow2->style
+		|| (flow1->style == HTML_CLUEFLOW_STYLE_LIST_ITEM && flow1->item_type != flow2->item_type)
+		|| !levels_equal (flow1, flow2)
+		|| !haligns_equal (HTML_CLUE (flow1)->halign, HTML_CLUE (flow2)->halign))) {
+		HTMLCursor *dest, *source;
+
+		if (html_clueflow_is_empty (HTML_CLUEFLOW (e->cursor->object->parent))) {
+			dest = html_cursor_dup (e->cursor);
+			source = html_cursor_dup (e->mark);
+		} else {
+			source = html_cursor_dup (e->cursor);
+			dest = html_cursor_dup (e->mark);
+		}
+
+		html_engine_selection_push (e);
+		html_engine_disable_selection (e);
+		html_cursor_jump_to_position_no_spell (e->cursor, e, dest->position);
+		html_engine_set_clueflow_style (e,
+						HTML_CLUEFLOW (source->object->parent)->style,
+						HTML_CLUEFLOW (source->object->parent)->item_type,
+						HTML_CLUE     (source->object->parent)->halign,
+						HTML_CLUEFLOW (source->object->parent)->levels->len,
+						HTML_CLUEFLOW (source->object->parent)->levels->data,
+						HTML_ENGINE_SET_CLUEFLOW_INDENTATION_ALL,
+						dir, TRUE);
+		html_engine_selection_pop (e);
+		html_cursor_destroy (source);
+		html_cursor_destroy (dest);
+	}
+}
+
 static gint
-delete_object_do (HTMLEngine *e, HTMLObject **object, guint *len)
+delete_object_do (HTMLEngine *e, HTMLObject **object, guint *len, HTMLUndoDirection dir, gboolean add_undo)
 {
 	GList *from, *to, *left, *right;
 	guint position;
@@ -424,6 +525,8 @@ delete_object_do (HTMLEngine *e, HTMLObject **object, guint *len)
 	level = prepare_delete_bounds (e, &from, &to, &left, &right);
 	place_cursor_before_mark (e);
 	move_cursor_before_delete (e);
+	if (add_undo)
+		check_flows (e, dir);
 	html_engine_disable_selection (e);
 	*len     = 0;
 	*object  = html_object_op_cut  (HTML_OBJECT (from->data), e, from->next, to->next, left, right, len);
@@ -458,14 +561,52 @@ check_table_1 (HTMLEngine *e)
 		html_cursor_forward (head, e);
 }
 
+static gboolean
+validate_tables (HTMLEngine *e, HTMLUndoDirection dir, gboolean add_undo, gboolean *fix_para)
+{
+	HTMLObject *next = html_object_next_not_slave (e->cursor->object);
+
+	*fix_para = FALSE;
+
+	if (next && HTML_IS_TABLE (next)) {
+		insert_empty_paragraph (e, dir, add_undo);
+		*fix_para = FALSE;
+
+		return TRUE;
+	} else if (!next) {
+		gint steps = 0;
+
+		while (html_cursor_forward (e->cursor, e)) {
+			steps ++;
+			if (HTML_IS_TABLE (e->cursor->object)) {
+				next = html_object_next_not_slave (e->cursor->object);
+				if (next) {
+					insert_empty_paragraph (e, dir, FALSE);
+					*fix_para = TRUE;
+					steps ++;
+					break;
+				}
+			} else
+				break;
+		}
+
+		if (steps)
+			html_cursor_backward_n (e->cursor, e, steps);
+	}
+
+	return FALSE;
+}
+
 static gint
 delete_object (HTMLEngine *e, HTMLObject **ret_object, guint *ret_len, HTMLUndoDirection dir, gboolean add_undo)
 {
 	html_engine_edit_selection_updater_update_now (e->selection_updater);
 	if (html_engine_is_selection_active (e)) {
 		HTMLObject *object;
-		guint len, position_before;
+		guint len, position_before, saved_position;
 		gint level;
+		gboolean backward;
+		gboolean fix_para;
 
 		if (!html_clueflow_is_empty (HTML_CLUEFLOW (e->cursor->object->parent))
 		    && !html_clueflow_is_empty (HTML_CLUEFLOW (e->mark->object->parent))) {
@@ -476,16 +617,27 @@ delete_object (HTMLEngine *e, HTMLObject **ret_object, guint *ret_len, HTMLUndoD
 			html_engine_disable_selection (e);
 			return 0;
 		}
+
 		position_before = MAX (e->cursor->position, e->mark->position);
-		level = delete_object_do (e, &object, &len);
+		level = delete_object_do (e, &object, &len, dir, add_undo);
 		if (ret_object && ret_len) {
-			*ret_object = html_object_op_copy (object, e, NULL, NULL, ret_len);
+			*ret_object = html_object_op_copy (object, NULL, e, NULL, NULL, ret_len);
 			*ret_len    = len;
 		}
+		backward = validate_tables (e, dir, add_undo, &fix_para);
+		if (fix_para) {
+			saved_position = e->cursor->position;
+			e->cursor->position = position_before + 1;
+			insert_setup_undo (e, 1, position_before, dir, FALSE, FALSE);
+			e->cursor->position = saved_position;
+		}
 		if (add_undo) {
-			delete_setup_undo (e, object, len, position_before, level, dir);
+			delete_setup_undo (e, object, len, position_before + (backward ? 1 : 0), level, dir);
 		} else
 			html_object_destroy (object);
+
+		if (backward)
+			html_cursor_backward (e->cursor, e);
 		gtk_html_editor_event (e->widget, GTK_HTML_EDITOR_EVENT_DELETE, NULL);
 
 		return level;
@@ -497,7 +649,9 @@ delete_object (HTMLEngine *e, HTMLObject **ret_object, guint *ret_len, HTMLUndoD
 void
 html_engine_delete (HTMLEngine *e)
 {
+	html_undo_level_begin (e->undo, "Delete", "Undelete");
 	delete_object (e, NULL, NULL, HTML_UNDO_UNDO, TRUE);
+	html_undo_level_end (e->undo);
 }
 
 gint
@@ -506,7 +660,9 @@ html_engine_cut (HTMLEngine *e)
 	gint rv;
 
 	html_engine_clipboard_clear (e);
+	html_undo_level_begin (e->undo, "Cut", "Uncut");
 	rv = delete_object (e, &e->clipboard, &e->clipboard_len, HTML_UNDO_UNDO, TRUE);
+	html_undo_level_end (e->undo);
 
 #ifdef OP_DEBUG
 	printf ("cut  len: %d\n", e->clipboard_len);
@@ -711,7 +867,7 @@ html_engine_paste (HTMLEngine *e)
 		HTMLObject *copy;
 		guint len = 0;
 
-		copy = html_object_op_copy (e->clipboard, e, NULL, NULL, &len);
+		copy = html_object_op_copy (e->clipboard, NULL, e, NULL, NULL, &len);
 		html_engine_paste_object (e, copy, e->clipboard_len);
 	}
 }
@@ -768,8 +924,8 @@ insert_empty_paragraph (HTMLEngine *e, HTMLUndoDirection dir, gboolean add_undo)
 		html_clue_append (HTML_CLUE (flow), e->cursor->object);
 	}
 
-	html_undo_level_begin (e->undo, "Insert paragraph", "Delete paragraph");
 	if (add_undo) {
+		html_undo_level_begin (e->undo, "Insert paragraph", "Delete paragraph");
 		insert_setup_undo (e, 1, position_before, dir, FALSE, FALSE);
 	}
 	g_list_free (left);
@@ -782,7 +938,8 @@ insert_empty_paragraph (HTMLEngine *e, HTMLUndoDirection dir, gboolean add_undo)
 	html_cursor_forward (e->cursor, e);
 	
 	gtk_html_editor_event_command (e->widget, GTK_HTML_COMMAND_INSERT_PARAGRAPH, FALSE);
-	html_undo_level_end (e->undo);
+	if (add_undo)
+		html_undo_level_end (e->undo);
 
 	html_engine_thaw (e);
 }
@@ -793,6 +950,77 @@ html_engine_insert_empty_paragraph (HTMLEngine *e)
 	insert_empty_paragraph (e, HTML_UNDO_UNDO, TRUE);
 }
 
+static char *picto_chars = "DO)(|/P\0:-\0:\0:-\0:\0:;=-\0:;\0:-~\0:\0:\0:-\0:\0:-\0:\0:-\0:\0";
+static gint picto_states [] = { 8, 13, 18, 26, 34, 39, 44, 0, -1, 11, 0, -1, 0, -2, 16, 0, -2, 0, -3, -4, -5, 23, 0, -3, -4, 0, -6, 30, 32, 0, -6, 0, -11, 0, -8, 37, 0, -8, 0, -9, 42, 0, -9, 0, -10, 47, 0, -10, 0};
+static gchar *picto_images [] = {
+	"smiley-1.png",
+	"smiley-2.png",
+	"smiley-3.png",
+	"smiley-4.png",
+	"smiley-5.png",
+	"smiley-6.png",
+	"smiley-7.png",
+	"smiley-8.png",
+	"smiley-9.png",
+	"smiley-10.png",
+	"smiley-11.png",
+};
+
+static void
+use_pictograms (HTMLEngine *e)
+{
+	gint pos;
+	gint state;
+	gint relative;
+	gunichar uc;
+
+	if (!html_object_is_text (e->cursor->object))
+		return;
+
+	pos = e->cursor->offset - 1;
+	state = 0;
+	while (pos >= 0) {
+		uc = html_text_get_char (HTML_TEXT (e->cursor->object), pos);
+		relative = 0;
+		while (picto_chars [state + relative]) {
+			if (picto_chars [state + relative] == uc)
+				break;
+			relative ++;
+		}
+		state = picto_states [state + relative];
+		/* 0 .. not found, -n .. found n-th */
+		if (state <= 0)
+			break;
+		pos --;
+	}
+
+	if (state < 0) {
+		HTMLObject *picto;
+		gchar *filename;
+		gchar *alt;
+		gint len;
+
+		if (pos > 0) {
+			uc = html_text_get_char (HTML_TEXT (e->cursor->object), pos - 1);
+			if (uc != ' ' && uc != '\t')
+				return;
+		}
+		/* printf ("found %d\n", -state); */
+		len = e->cursor->offset  - pos;
+		alt = g_strndup (html_text_get_text (HTML_TEXT (e->cursor->object), pos), len);
+		html_cursor_backward_n (e->cursor, e, len);
+		html_engine_set_mark (e);
+		html_cursor_forward_n (e->cursor, e, len);
+
+		filename = g_strconcat ("file://" ICONDIR "/", picto_images [-state - 1], NULL);
+		picto = html_image_new (e->image_factory, filename, NULL, NULL, -1, -1, FALSE, FALSE, 0, NULL,
+					HTML_VALIGN_MIDDLE, FALSE);
+		html_image_set_alt (HTML_IMAGE (picto), alt);
+		g_free (alt);
+		html_engine_paste_object (e, picto, html_object_get_length (picto));
+	}
+}
+
 void
 html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
 {
@@ -833,6 +1061,8 @@ html_engine_insert_text (HTMLEngine *e, const gchar *text, guint len)
 			}
 			insert_object (e, o, html_object_get_length (o), e->cursor->position + html_object_get_length (o),
 				       1, HTML_UNDO_UNDO, check);
+			if (alen == 1 && !HTML_IS_PLAIN_PAINTER (e->painter) && GTK_HTML_PROPERTY (e->widget, magic_smileys))
+				use_pictograms (e);
 		}
 		if (nl) {
 			html_engine_insert_empty_paragraph (e);
@@ -960,7 +1190,7 @@ html_engine_set_insertion_link (HTMLEngine *e, const gchar *url, const gchar *ta
 }
 
 void
-html_engine_insert_link (HTMLEngine *e, const gchar *url, const gchar *target)
+html_engine_edit_set_link (HTMLEngine *e, const gchar *url, const gchar *target)
 {
 	if (html_engine_is_selection_active (e)) {
 		HTMLEngineLinkInsertData data;
@@ -1033,6 +1263,24 @@ html_engine_append_object (HTMLEngine *e, HTMLObject *o, guint len)
 	html_undo_level_end (e->undo);
 }
 
+static void
+replace_objects_in_clue_from_another (HTMLClue *dest, HTMLClue *src)
+{
+	HTMLObject *cur, *next;
+
+	for (cur = dest->head; cur; cur = next) {
+		next = cur->next;
+		html_object_remove_child (cur->parent, cur);
+		html_object_destroy (cur);
+	}
+
+	for (cur = src->head; cur; cur = next) {
+		next = cur->next;
+		html_object_remove_child (cur->parent, cur);
+		html_clue_append (dest, cur);
+	}
+}
+
 static void
 append_flow (HTMLEngine *e, HTMLObject *o, guint len, HTMLUndoDirection dir)
 {
@@ -1047,19 +1295,28 @@ append_flow (HTMLEngine *e, HTMLObject *o, guint len, HTMLUndoDirection dir)
 	g_return_if_fail (html_clueflow_is_empty (HTML_CLUEFLOW (e->cursor->object->parent)));
 
 	where = e->cursor->object->parent;
+	html_object_change_set (o, HTML_CHANGE_ALL_CALC);
 
 	e->cursor->object = html_object_get_head_leaf (o);
 	e->cursor->offset = 0;
+
+	/* be sure we have valid cursor position (like when there is a focusable container) */
 	position = e->cursor->position;
 	while (html_cursor_backward (e->cursor, e))
 		;
 	e->cursor->position = position;
-	html_clue_append_after (HTML_CLUE (where->parent), o, where);
-	html_object_remove_child (where->parent, where);
-	html_object_destroy (where);
+
+	/* we move objects between flows to preserve attributes as indentation, ... */
+	if (HTML_IS_CLUEFLOW (o)) {
+		replace_objects_in_clue_from_another (HTML_CLUE (where), HTML_CLUE (o));
+		html_object_destroy (o);
+	} else {
+		html_clue_append_after (HTML_CLUE (where->parent), o, where);
+		html_object_remove_child (where->parent, where);
+		html_object_destroy (where);
+	}
 
 	html_cursor_forward_n (e->cursor, e, len);
-	html_object_change_set (o, HTML_CHANGE_ALL_CALC);
 	html_engine_thaw (e);
 
 	insert_setup_undo (e, len, position_before, dir, FALSE, FALSE);
