@@ -26,21 +26,24 @@
 
 #include <config.h>
 #include <ctype.h>
+#include <string.h>
 #include <glib.h>
 
 #include "gtkhtml.h"
 #include "gtkhtml-properties.h"
 
-#include "htmlobject.h"
 #include "htmlclueflow.h"
+#include "htmlcolorset.h"
 #include "htmlcursor.h"
 #include "htmllinktext.h"
+#include "htmlobject.h"
 #include "htmltable.h"
 #include "htmltext.h"
 #include "htmltextslave.h"
 #include "htmlimage.h"
 #include "htmlinterval.h"
 #include "htmlselection.h"
+#include "htmlsettings.h"
 #include "htmlundo.h"
 
 #include "htmlengine-edit.h"
@@ -325,7 +328,34 @@ html_engine_clipboard_clear (HTMLEngine *e)
 HTMLObject *
 html_engine_new_text (HTMLEngine *e, const gchar *text, gint len)
 {
-	return html_text_new_with_len (text, len, e->insertion_font_style, e->insertion_color);
+	if (e->insertion_url && *e->insertion_url) {
+		return html_link_text_new_with_len (text, len, e->insertion_font_style, e->insertion_color,
+						    e->insertion_url, e->insertion_target);
+	} else
+		return html_text_new_with_len (text, len, e->insertion_font_style, e->insertion_color);
+}
+
+HTMLObject *
+html_engine_new_link (HTMLEngine *e, const gchar *text, gint len, gchar *url)
+{
+	HTMLObject *link;
+	gchar *real_url, *real_target;
+
+	real_target = strchr (text, '#');
+	if (real_target) {
+		real_url = g_strndup (url, real_target - url);
+		real_target ++;
+	} else
+		real_url = url;
+		
+	link = html_link_text_new_with_len (text, len, e->insertion_font_style,
+					    html_colorset_get_color (e->settings->color_set, HTMLLinkColor),
+					    real_url, real_target);
+
+	if (real_target)
+		g_free (real_url);
+
+	return link;
 }
 
 HTMLObject *
@@ -353,7 +383,7 @@ html_engine_get_indent (HTMLEngine *e)
 
 	return e->cursor->object && e->cursor->object->parent
 		&& HTML_OBJECT_TYPE (e->cursor->object->parent) == HTML_TYPE_CLUEFLOW
-		? HTML_CLUEFLOW (e->cursor->object->parent)->level : 0;
+		? html_clueflow_get_indentation (HTML_CLUEFLOW (e->cursor->object->parent)) : 0;
 }
 
 #define LINE_LEN 71
@@ -660,3 +690,12 @@ html_engine_prev_cell (HTMLEngine *e)
 
 	return FALSE;
 }
+
+void
+html_engine_set_title (HTMLEngine *e, const gchar *title)
+{
+	if (e->title)
+		g_string_free (e->title, TRUE);
+	e->title = g_string_new (title);
+	gtk_signal_emit_by_name (GTK_OBJECT (e), "title_changed");
+}
